@@ -2,30 +2,57 @@ use crate::{
     Rounding,
     ListingCurrencies,
     helpers,
+    error::{TryFromListingCurrenciesError, ParseError},
+    traits::SerializeCurrencies,
     constants::{
         KEYS_SYMBOL,
         KEY_SYMBOL,
         METAL_SYMBOL,
+        EMPTY_SYMBOL,
     },
 };
-use std::{fmt, ops::{self, AddAssign, SubAssign}};
+use std::{fmt, cmp::{Ord, Ordering}, ops::{self, AddAssign, SubAssign, MulAssign, DivAssign}};
 use serde::{Serialize, Deserialize, Serializer, Deserializer, de::Error, ser::SerializeStruct};
 
-/// Currencies.
-/// 
-/// The keys and metal values for `Currencies` are defined as an i32. Metal values 
-/// are stored as their lowest denomination, 1 weapon. A metal value of 6 would be 
-/// equivalent to 3 scrap. You may use the `ONE_REF`, `ONE_REC`, `ONE_SCRAP`, 
-/// and `ONE_WEAPON` constants to perform arithmatic.
+/// For storing item currencies values.
 #[derive(Debug, Serialize, Deserialize, Eq, PartialEq, Clone)]
 #[serde(remote = "Self")]
 pub struct Currencies {
     #[serde(default)]
+    /// Amount of keys.
     pub keys: i32,
     #[serde(deserialize_with = "helpers::metal_deserializer", default)]
+    /// Amount of metal expressed as weapons. A metal value of 6 would 
+    /// be equivalent to 3 scrap. You may use the `ONE_REF`, `ONE_REC`, `ONE_SCRAP`, and 
+    /// `ONE_WEAPON`constants to perform arithmatic.
     pub metal: i32,
 }
 
+impl PartialOrd for Currencies {
+    fn partial_cmp(&self, other: &Currencies) -> Option<Ordering> {
+       Some(self.cmp(other))
+    }
+}
+
+impl Ord for Currencies {
+    
+    fn cmp(&self, other:&Self) -> Ordering {
+        if self.keys > other.keys {
+            Ordering::Greater
+        } else if self.keys < other.keys {
+            Ordering::Less
+        } else if self.metal > other.metal {
+            Ordering::Greater
+        } else if self.metal < other.metal {
+            Ordering::Less
+        } else {
+            Ordering::Equal
+        }
+    }
+}
+
+impl SerializeCurrencies for Currencies {}
+
 impl Default for Currencies {
     
     fn default() -> Self {
@@ -42,7 +69,19 @@ impl Currencies {
         }
     }
     
-    /// Creates currencies from a metal value using the given key price.
+    /// Converts a metal value into the appropriate number of keys using the given key price 
+    /// (represented as weapons).
+    /// 
+    /// # Examples
+    ///
+    /// ```
+    /// use tf2_price::{Currencies, refined, scrap};
+    /// 
+    /// assert_eq!(
+    ///     Currencies::from_metal(refined!(80), refined!(60)),
+    ///     Currencies { keys: 1, metal: refined!(20) },
+    /// );
+    /// ```
     pub fn from_metal(metal: i32, key_price: i32) -> Self {
         Self {
             // Will be 0 if metal is 30 and key price is 32 (rounds down)
@@ -51,7 +90,7 @@ impl Currencies {
         }
     }
     
-    /// Converts from `ListingCurrencies` using the given key price.
+    /// Converts from `ListingCurrencies` using the given key price (represented as weapons).
     pub fn from_listing_currencies(currencies: ListingCurrencies, key_price: i32) -> Self {
         let keys = currencies.keys;
         let metal = currencies.metal;
@@ -62,7 +101,8 @@ impl Currencies {
         }
     }
     
-    /// Converts an f32 key value into `Currencies` using the given key price.
+    /// Converts an f32 key value into `Currencies` using the given key price represented as 
+    /// weapons.
     pub fn from_keys_f32(keys: f32, key_price: i32) -> Self {
         Self {
             keys: keys as i32,
@@ -70,7 +110,7 @@ impl Currencies {
         }
     }
     
-    /// Converts `Currencies` into a metal value using the given key price.
+    /// Converts currencies to a metal value using the given key price (represented as weapons).
     pub fn to_metal(&self, key_price: i32) -> i32 {
         self.metal + (self.keys * key_price)
     }
@@ -86,7 +126,7 @@ impl Currencies {
     }
 }
 
-/// Comparson with `ListingCurrencies` will fail if `ListingCurrencies` has a fractional key value.
+/// Comparison with `ListingCurrencies` will fail if `ListingCurrencies` has a fractional key value.
 impl PartialEq<ListingCurrencies> for Currencies {
     
     fn eq(&self, other: &ListingCurrencies) -> bool {
@@ -170,8 +210,40 @@ impl SubAssign<&Currencies> for Currencies {
     }
 }
 
+impl MulAssign<i32> for Currencies {
+    
+    fn mul_assign(&mut self, other: i32) {
+        self.keys *= other;
+        self.metal *= other;
+    }
+}
+
+impl MulAssign<f32> for Currencies {
+    
+    fn mul_assign(&mut self, other: f32) {
+        self.keys = (self.keys as f32 * other).round() as i32;
+        self.metal = (self.metal as f32 * other).round() as i32;
+    }
+}
+
+impl DivAssign<i32> for Currencies {
+    
+    fn div_assign(&mut self, other: i32) {
+        self.keys /= other;
+        self.metal /= other;
+    }
+}
+
+impl DivAssign<f32> for Currencies {
+    
+    fn div_assign(&mut self, other: f32) {
+        self.keys = (self.keys as f32 / other).round() as i32;
+        self.metal = (self.metal as f32 / other).round() as i32;
+    }
+}
+
 impl<'a> TryFrom<&'a str> for Currencies {
-    type Error = &'static str;
+    type Error = ParseError;
     
     fn try_from(string: &'a str) -> Result<Self, Self::Error>  {
         let (keys, metal) = helpers::parse_from_string::<i32>(string)?;
@@ -185,11 +257,11 @@ impl<'a> TryFrom<&'a str> for Currencies {
 
 /// Results in error if `ListingCurrencies` contains a fractional key value.
 impl TryFrom<ListingCurrencies> for Currencies {
-    type Error = &'static str;
+    type Error = TryFromListingCurrenciesError;
     
     fn try_from(currencies: ListingCurrencies) -> Result<Self, Self::Error> {
         if currencies.is_fract() {
-            return Err("Currencies contain fractional key value");
+            return Err(TryFromListingCurrenciesError { fract: currencies.keys.fract() });
         }
         
         Ok(Currencies {
@@ -218,7 +290,7 @@ impl TryFrom<&ListingCurrencies> for Currencies {
 impl fmt::Display for Currencies {
     
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        if self.keys > 0 && self.metal > 0 {
+        if self.keys != 0 && self.metal != 0 {
             write!(
                 f,
                 "{} {}, {} {}",
@@ -227,14 +299,14 @@ impl fmt::Display for Currencies {
                 helpers::get_metal_float(self.metal),
                 METAL_SYMBOL,
             )
-        } else if self.keys > 0 {
+        } else if self.keys != 0 {
             write!(
                 f,
                 "{} {}",
                 self.keys,
                 helpers::pluralize(self.keys, KEY_SYMBOL, KEYS_SYMBOL),
             )
-        } else if self.metal > 0 {
+        } else if self.metal != 0 {
             write!(
                 f,
                 "{} {}",
@@ -242,7 +314,7 @@ impl fmt::Display for Currencies {
                 METAL_SYMBOL,
             )
         } else {
-            write!(f, "")
+            write!(f, "{}", EMPTY_SYMBOL)
         }
     }
 }
@@ -394,6 +466,66 @@ mod tests {
         });
     }
     
+    #[test]
+    fn currencies_mul_assign_i32() {
+        let mut currencies = Currencies {
+            keys: 10,
+            metal: refined!(10),
+        };
+        
+        currencies *= 2;
+        
+        assert_eq!(currencies, Currencies {
+            keys: 20,
+            metal: refined!(20),
+        });
+    }
+    
+    #[test]
+    fn currencies_mul_assign_f32() {
+        let mut currencies = Currencies {
+            keys: 10,
+            metal: refined!(10),
+        };
+        
+        currencies *= 2.5;
+        
+        assert_eq!(currencies, Currencies {
+            keys: 25,
+            metal: refined!(25),
+        });
+    }
+    
+    #[test]
+    fn currencies_div_assign_i32() {
+        let mut currencies = Currencies {
+            keys: 10,
+            metal: refined!(10),
+        };
+        
+        currencies /= 2;
+        
+        assert_eq!(currencies, Currencies {
+            keys: 5,
+            metal: refined!(5),
+        });
+    }
+    
+    #[test]
+    fn currencies_div_assign_f32() {
+        let mut currencies = Currencies {
+            keys: 10,
+            metal: refined!(10),
+        };
+        
+        currencies /= 2.5;
+        
+        assert_eq!(currencies, Currencies {
+            keys: 4,
+            metal: refined!(4),
+        });
+    }
+    
     #[test]
     fn parses_currencies_from_string() {
         let currencies = Currencies::try_from("2 keys, 23.44 ref").unwrap();
@@ -756,4 +888,41 @@ mod tests {
             expected,
         );
     }
+    
+    #[test]
+    fn accepts_trait_currencies() {
+        fn get_keys<T>(currencies: &T) -> String
+        where T: SerializeCurrencies {
+            serde_json::to_string(currencies).unwrap()
+        }
+        
+        let currencies = Currencies { keys: 1, metal: 1 };
+        let serialized = get_keys(&currencies);
+        
+        assert_eq!(serialized, r#"{"keys":1,"metal":0.05}"#);
+    }
+    
+    #[test]
+    fn greater_than() {
+        assert!(Currencies { keys: 1, metal: 5 } > Currencies { keys: 0, metal: 10});
+    }
+    
+    #[test]
+    fn less_than() {
+        assert!(Currencies { keys: 0, metal: 1 } < Currencies { keys: 0, metal: 4});
+    }
+    
+    #[test]
+    fn sorts() {
+        let mut currencies = vec![
+            Currencies { keys: 2, metal: 4},
+            Currencies { keys: 0, metal: 2},
+            Currencies { keys: 10, metal: 4},
+        ];
+        
+        // lowest to highest
+        currencies.sort();
+        
+        assert_eq!(*currencies.iter().rev().next().unwrap(), Currencies { keys: 10, metal: 4});
+    }
 }
\ No newline at end of file
