@@ -2,19 +2,20 @@ use crate::{
     Rounding,
     Currencies,
     helpers,
+    error::ParseError,
+    traits::SerializeCurrencies,
     constants::{
         KEYS_SYMBOL,
         KEY_SYMBOL,
         METAL_SYMBOL,
+        EMPTY_SYMBOL,
     },
 };
-use std::{fmt, ops::{self, AddAssign, SubAssign}};
+use std::{fmt, cmp::{Ord, Ordering}, ops::{self, AddAssign, SubAssign, MulAssign, DivAssign}};
 use serde::{Serialize, Deserialize, Serializer, Deserializer, de::Error, ser::SerializeStruct};
 
-/// Currencies for listings.
-/// 
-/// The `keys` field for `ListingCurrencies` is defined as an f32. Use this anywhere you
-/// may need key values which include decimal places.
+/// The `keys` field for `ListingCurrencies` is defined as an f32. Use this anywhere you may
+/// need key values which include decimal places.
 #[derive(Debug, Serialize, Deserialize, PartialEq, Clone)]
 #[serde(remote = "Self")]
 pub struct ListingCurrencies {
@@ -24,6 +25,33 @@ pub struct ListingCurrencies {
     pub metal: i32,
 }
 
+impl PartialOrd for ListingCurrencies {
+    fn partial_cmp(&self, other: &ListingCurrencies) -> Option<Ordering> {
+       Some(self.cmp(other))
+    }
+}
+
+impl Ord for ListingCurrencies {
+    
+    fn cmp(&self, other:&Self) -> Ordering {
+        if self.keys > other.keys {
+            Ordering::Greater
+        } else if self.keys < other.keys {
+            Ordering::Less
+        } else if self.metal > other.metal {
+            Ordering::Greater
+        } else if self.metal < other.metal {
+            Ordering::Less
+        } else {
+            Ordering::Equal
+        }
+    }
+}
+
+impl Eq for ListingCurrencies {}
+
+impl SerializeCurrencies for ListingCurrencies {}
+
 impl Default for ListingCurrencies {
     
     fn default() -> Self {
@@ -45,7 +73,7 @@ impl ListingCurrencies {
         self.keys.fract() != 0.0
     }
     
-    /// Creates currencies from a metal value using the given key price.
+    /// Converts currencies to a metal value using the given key price (represented as weapons).
     pub fn to_metal(&self, key_price: i32) -> i32 {
         self.metal + (self.keys * key_price as f32) as i32
     }
@@ -206,8 +234,40 @@ impl SubAssign<&Currencies> for ListingCurrencies {
     }
 }
 
+impl MulAssign<i32> for ListingCurrencies {
+    
+    fn mul_assign(&mut self, other: i32) {
+        self.keys *= other as f32;
+        self.metal *= other;
+    }
+}
+
+impl MulAssign<f32> for ListingCurrencies {
+    
+    fn mul_assign(&mut self, other: f32) {
+        self.keys *= other;
+        self.metal = (self.metal as f32 * other).round() as i32;
+    }
+}
+
+impl DivAssign<i32> for ListingCurrencies {
+    
+    fn div_assign(&mut self, other: i32) {
+        self.keys /= other as f32;
+        self.metal /= other;
+    }
+}
+
+impl DivAssign<f32> for ListingCurrencies {
+    
+    fn div_assign(&mut self, other: f32) {
+        self.keys /= other;
+        self.metal = (self.metal as f32 / other).round() as i32;
+    }
+}
+
 impl<'a> TryFrom<&'a str> for ListingCurrencies {
-    type Error = &'static str;
+    type Error = ParseError;
     
     fn try_from(string: &'a str) -> Result<Self, Self::Error>  {
         let (keys, metal) = helpers::parse_from_string::<f32>(string)?;
@@ -242,7 +302,7 @@ impl From<&Currencies> for ListingCurrencies {
 impl fmt::Display for ListingCurrencies {
     
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        if self.keys > 0.0 && self.metal > 0 {
+        if self.keys != 0.0 && self.metal != 0 {
             write!(
                 f,
                 "{} {}, {} {}",
@@ -251,14 +311,14 @@ impl fmt::Display for ListingCurrencies {
                 helpers::get_metal_float(self.metal),
                 METAL_SYMBOL,
             )
-        } else if self.keys > 0.0 {
+        } else if self.keys != 0.0 {
             write!(
                 f,
                 "{} {}",
                 helpers::print_float(self.keys),
                 helpers::pluralize_float(self.keys, KEY_SYMBOL, KEYS_SYMBOL),
             )
-        } else if self.metal > 0 {
+        } else if self.metal != 0 {
             write!(
                 f,
                 "{} {}",
@@ -266,7 +326,7 @@ impl fmt::Display for ListingCurrencies {
                 METAL_SYMBOL,
             )
         } else {
-            write!(f, "")
+            write!(f, "{}", EMPTY_SYMBOL)
         }
     }
 }
@@ -558,4 +618,28 @@ mod tests {
             expected,
         );
     }
+    
+    #[test]
+    fn greater_than() {
+        assert!(ListingCurrencies { keys: 1.0, metal: 5 } > ListingCurrencies { keys: 0.0, metal: 10});
+    }
+    
+    #[test]
+    fn less_than() {
+        assert!(ListingCurrencies { keys: 0.0, metal: 1 } < ListingCurrencies { keys: 0.0, metal: 4});
+    }
+    
+    #[test]
+    fn sorts() {
+        let mut currencies = vec![
+            ListingCurrencies { keys: 2.0, metal: 4},
+            ListingCurrencies { keys: 0.0, metal: 2},
+            ListingCurrencies { keys: 10.0, metal: 4},
+        ];
+        
+        // lowest to highest
+        currencies.sort();
+        
+        assert_eq!(*currencies.iter().rev().next().unwrap(), ListingCurrencies { keys: 10.0, metal: 4});
+    }
 }
\ No newline at end of file
